#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <QColorDialog>
#include <unistd.h>
#include <iostream>
#include <stdio.h>
#include <termios.h>
#include <QListWidgetItem>
#include <QSettings>
#include <QTime>
#include <QProcess>
#include <QSignalMapper>
#include "alarmtime.h"
#include "microcontroller.h"


namespace Ui
{
class MainWindow;
}

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    explicit MainWindow(QSettings *settings, Microcontroller *micro, bool isRemoteMode = false, QWidget *parent = 0);
    ~MainWindow();



private:
    Ui::MainWindow *ui;

    QColorDialog colorChooser;

    QSettings *_settings;

    Microcontroller *_micro;

    void remoteMode();

signals:

    void signalAmpOn();
    void signalAmpOff();

    void signalAlmNightStateChanged(int state);
    void signalAlmNightChanged(const QTime time);

    void signalAlmAlarmStateChanged(int state);
    void signalAlmAlarmChanged(const QTime time);

    void showAdvRelayDialog();

private slots:

     void postActivate();

    //RGB
    void slotChangedRgb(const QColor color);
    void slotApplyPreset();
    void slotAmpChkbtn(int state);

    void changeHeaderLableText(const QString string);

    //Relays
    void slotAmpToggle(int state);
    void slotBSpeakerToggle(int state);
    void slotBSpeakerAutoToggle(int state);
    void slotInfMirrorToggle(int state);
    void slotInfMirrorAutoToggle(int state);
    void slotAirconToggle(int state);

    //Alarm
    void slotChangedAlarmTime(const QTime time);
    void saveAlarmState(int state);

    //Night
    void slotChangedNightTime(const QTime time);

    //syncoff

    void slotSyncoff();

};

#endif // MAINWINDOW_H
